# edge-cni设计

edge-cni负责边缘节点的容器网络管理

1. IPSec配置生成
2. 容器网络管理

## 1. IPSec配置生成

IPSec用来解决云边通信以及边边通信

 * 边缘节点跟云端的通信会通过云端的一个路由来实现, 这需要边缘端配置相应的管道
 * 边边通信会假设需要通信的节点都属于一个社区，因此同属一个社区的节点会有一个`fabedge/edge-community`， 每个边缘端会配置跟同社区节点的管道。如果一个节点没有`bocloud.com/edge-community`标签，那么不能跟其他边缘节点通信。

因为边缘节点的不稳定性，导致节点可能随时退出或加入，节点的IP也可能随时会变动，因此需要每个边缘节点起一个服务, 这个服务会监听自身节点及该节点所在社区的节点信息变动，并根据最新信息生成IPSec配置

## 2. 容器网络管理

当edgecore在边缘节点创建Pod时,调用edge-cni插件将容器加入网络，当Pod被删除时，又需要调用edge-cni插件来收回或销毁之前分配的资源

### 加入网络

1. 检查本地IP网段是否够用，不够需要向云端申请新的IP段参考前面的边缘节点IP网段管理
2. 在节点创建一个网桥
3. 在容器内部创建veth pair, 并将其中一个veth放到主机命名空间，跟网桥连接
4. 检查本地IP资源是否足够，不足向云端申请一个IP段, 
5. 然后获取一个IP地址，为容器veth分配IP地址, 配置网关和路由信息
6. 在容器内发起一个ARP广播，通知子网其他设备IP和MAC的映射更改

![cmdAdd](./cni-cmd-add.png)

### 退出网络

1. 删除容器veth
2. 回收IP地址

![cmdDel](./cni-cmd-del.png)

### IP网段申请和回收:

* 申请. edge-cni在执行时先判断本地网段是否足够，如果不够，则向云端申请一个网段.
* 回收. 为了不浪费IP网段，需要有一定的回收能力。

为了实现IP网段的申请和回收功能，需要在云端实现一个IPController服务，该服务向边缘节点提供网段申请功能，同时记录节点和网段的映射关系。当边缘节点没有注销时，相关的网段不会释放，只有在节点被注销后才会释放。


*注： 因为cni插件的调用是启动进程来实现的，为了减少不必要的内存波动，这里将网桥和IP分配的功能实现在一起*