// Copyright 2021 BoCloud
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package v1alpha1

import (
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

type CommunitySpec struct {
	Members []string `json:"members,omitempty"`
}

// Community is used to manage a communication unit, it's members
// should be edge nodes
// +kubebuilder:object:root=true
// +kubebuilder:resource:shortName=cmm
// +kubebuilder:resource:scope=Cluster
// +kubebuilder:printcolumn:name="Members",type="string",JSONPath=".spec.members",description="community members"
// +kubebuilder:printcolumn:name="Age",type="date",JSONPath=".metadata.creationTimestamp",description="How long a community is created"
type Community struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec CommunitySpec `json:"spec,omitempty"`
}

// CommunityList contains a list of Community
// +kubebuilder:object:root=true
type CommunityList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []Community `json:"items"`
}
